import acm.program.*;
import acm.util.ErrorException;
import acm.util.RandomGenerator;

import java.awt.event.*;
import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;

import javax.swing.*;

public class WikiWalker extends ConsoleProgram {
	/* The number of columns in the text box. */
	private static final int NUM_COLUMNS = 25;
	
	/* The file containing the names of the articles. */
	private static final String ARTICLES_FILE = "articles.txt";
	
	/* The file containing the links between the articles. */
	private static final String LINKS_FILE = "links.txt";
	
	/* Data about the Wikipedia graph. */
	private ArrayList<String> articles;
	private HashMap<String, ArrayList<String>> links;
	
	/* The currently-selected page. */
	private String currPage;
	
	/* A text field storing the page that the user has opted to warp to. */
	private JTextField selection;
	
	public void init() {
		setFont("DejaVuSerif-BOLD-24");
		
		/* Load the articles and start at Article 0. */
		articles = readArticleNames();
		links    = readLinks(articles);
		currPage = articles.get(0);
		println(currPage);
		
		/* Set up interactors. */
		add(new JButton("Random Link"), SOUTH);
		add(new JButton("Random Page"), SOUTH);
		add(new JLabel("Destination: "), SOUTH);
		selection = new JTextField(NUM_COLUMNS);
		add(selection, SOUTH);
		add(new JButton("Go!"), SOUTH);
		
		/* Enable events. */
		addActionListeners();
		selection.addActionListener(this);
		selection.setActionCommand("Go!");
	}
	
	/**
	 * Given a list of possible destination pages, chooses a random destination
	 * page and warps to that page.
	 * 
	 * @param options The list of possible page options, which must be nonempty.
	 */
	private void warpToPage(ArrayList<String> options) {
		RandomGenerator rgen = RandomGenerator.getInstance();
		currPage = options.get(rgen.nextInt(0, options.size() - 1));
		println(currPage);
	}
	
	public void actionPerformed(ActionEvent e) {
		/* If the user chooses "Random Link," follow a randomly-chosen link from the current
		 * page.
		 */
		if (e.getActionCommand().equals("Random Link")) {
			warpToPage(links.get(currPage));
		}
		/* If the user chooses "Random Page," jump to a totally random page. */
		else if (e.getActionCommand().equals("Random Page")) {
			warpToPage(articles);
		}
		/* If the user chooses "Go!", bring them to the page they requested, if possible. */
		else if (e.getActionCommand().equals("Go!")) {
			String dest = selection.getText();
			if (links.containsKey(dest)) {
				currPage = dest;
				println(currPage);
			} else {
				showErrorMessage("Sorry, I don't know that page.");
			}
		}
	}
	
	/**
	 * Reads the articles file and returns a list of the names in the file.
	 * 
	 * @return The list of article titles, in order.
	 */
	private ArrayList<String> readArticleNames() {
		try {
			BufferedReader br = new BufferedReader(new FileReader(ARTICLES_FILE));
			ArrayList<String> result = new ArrayList<String>();
			
			/* Use the standard file-reading loop to populate the articles list. */
			while (true) {
				String title = br.readLine();
				if (title == null) break;
				
				result.add(title);
			}
			
			br.close();
			
			return result;
			
		} catch (IOException e) {
			throw new ErrorException(e);
		}
	}
	
	/**
	 * Builds up a graph of the articles in Wikipedia.  Since the links are
	 * stored as pairs of integers, we need the article titles list to map
	 * them back to their names.
	 * 
	 * @param articleTitles The list of all article titles, in order.
	 * @return A graph of the links in Wikipedia.
	 */
	private HashMap<String, ArrayList<String>> readLinks(ArrayList<String> articleTitles) {
		try {
			BufferedReader br = new BufferedReader(new FileReader(LINKS_FILE));
			HashMap<String, ArrayList<String>> result = new HashMap<String, ArrayList<String>>();
			
			while (true) {
				/* Read the start and end pages, stopping if we're done. */
				String startString = br.readLine();
				String endString = br.readLine();
				if (endString == null) break;
				
				/* Convert from the string representation of the link to an
				 * integer representation of the link.
				 */
				int startIndex = Integer.parseInt(startString);
				int endIndex = Integer.parseInt(endString);
				
				/* Map from those integers to the corresponding pages. */
				String startPage = articleTitles.get(startIndex);
				String endPage = articleTitles.get(endIndex);
				
				/* Add this link to the graph.  Ensure that there is a list
				 * associated with the title, then add it in.
				 */
				if (!result.containsKey(startPage)) {
					result.put(startPage,  new ArrayList<String>());
				}		
				result.get(startPage).add(endPage);
			}
			
			br.close();
			
			/* See how many links there are. */
			return result;
			
		} catch (IOException e) {
			throw new ErrorException(e);
		}
	}
}
